﻿using UnityEngine;
using System.Collections;
using MathExtension;

public class Button : MonoBehaviour {

	public string		group_name;
	
	public struct StateBit {

		public bool	trigger_on;

		public bool	previous;
		public bool	current;

		public void	update_trigger()
		{
			this.trigger_on = (!this.previous && this.current);
		}
	};

	public StateBit		focused;
	public StateBit		pressed;

	public Sprite2DControl	sprite = null;

	protected float		scale_timer = 0.0f;

	protected bool		is_visible = true;
	protected bool		is_active  = true;

	public delegate	void	Func(string name);
	public Func		on_trigger_pressed = (name)=>{};

	protected struct PressingAction {

		public bool		is_active;
		public bool		press_down;
	};
	protected PressingAction	press_act;

	// ================================================================ //
	// MonoBehaviour からの継承.

	public void		setActive(bool is_active)
	{
		this.is_active = is_active;

		if(this.is_active) {

			this.sprite.setVertexColor(Color.white);

		} else {

			this.sprite.setVertexColor(new Color(0.7f, 0.7f, 0.7f));
		}
	}

	void	Awake()
	{
		this.reset();
	}
	
	void	Start()
	{
	}
	
	void	Update()
	{
	}

	public void		reset()
	{
		this.focused.trigger_on = false;
		this.focused.previous   = false;
		this.focused.current    = false;

		this.pressed = this.focused;

		this.press_act.is_active = false;
		this.press_act.press_down = false;

		this.scale_timer = 0.0f;

		if(this.sprite != null) {

			this.sprite.setScale(Vector2.one);
		}
	}

	public void		setVisible(bool is_visible)
	{
		this.is_visible = is_visible;
		this.sprite.setVisible(this.is_visible);

		if(!this.is_visible) {

			this.reset();
		}
	}

	public void		update(bool is_focusable)
	{
		if(this.is_visible) {

			this.update_entity(is_focusable);
		}
	}

	protected void	update_entity(bool is_focusable)
	{
		this.focused.previous = this.focused.current;
		
		// フォーカス（ロールオーバー）.

		Vector2		mouse_pos = Input.mousePosition.xy();

		mouse_pos = Sprite2DRoot.get().convertMousePosition(mouse_pos);

		this.focused.current = false;

		if(this.is_active) {

			if(is_focusable) {

				if(this.sprite.isContainPoint(mouse_pos)) {
	
					this.focused.current = true;	
				}
			}

		} else {

			if(this.focused.previous) {

				if(is_focusable) {
	
					if(this.sprite.isContainPoint(mouse_pos)) {
		
						this.focused.current = true;	
					}
				}
			}
		}


		this.focused.update_trigger();

		// クリック.

		this.pressed.previous = this.pressed.current;
		this.pressed.current  = false;
		
		if(Input.GetMouseButton(0)) {
	
			if(this.is_active) {

				if(this.focused.current) {
	
					this.pressed.current = true;
				}
			}
		}

		// 押した演出中は、押されている状態にする.
		if(this.press_act.is_active) {

			this.pressed.current = true;
		}

		this.pressed.update_trigger();

		if(this.pressed.trigger_on) {

			this.on_trigger_pressed(this.name);

			this.press_act.is_active  = true;
			this.press_act.press_down = true;
		}

		//

		if(this.press_act.is_active) {

			if(this.press_act.press_down) {

				this.scale_timer -= Time.deltaTime;

				if(this.scale_timer <= 0.0f) {

					this.scale_timer = 0.0f;

					this.press_act.press_down = false;
				}

			} else {

				this.scale_timer += Time.deltaTime;

				if(this.scale_timer >= 0.1f) {

					this.scale_timer = 0.1f;
					this.press_act.is_active = false;
				}
			}

			this.sprite.setScale(Vector2.one*Mathf.Lerp(1.0f, 1.2f, this.scale_timer/0.1f));

		} else {

			if(this.focused.current) {
				
				this.scale_timer += Time.deltaTime;
				
			} else {
				
				this.scale_timer -= Time.deltaTime;
			}

			this.scale_timer = Mathf.Clamp(this.scale_timer, 0.0f, 0.1f);

			this.sprite.setScale(Vector2.one*Mathf.Lerp(1.0f, 1.2f, this.scale_timer/0.1f));
		}

		if(this.focused.current) {

			this.sprite.setDepth(ButtonRoot.FORCUSED_BUTTON_DEPTH);

		} else {

			this.sprite.setDepth(9.0f);
		}
	}

	// ================================================================ //

	public void		create(Texture texture, Vector2 size)
	{
		this.sprite = Sprite2DRoot.get().createSprite(texture, 2, true);
		this.sprite.setSize(size);
	}	
}
