﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class ButtonRoot : MonoBehaviour {

	public class Group {

		public string		name = "";
		public bool			is_visible = true;

		public Button		focused_button = null;
		public List<Button>	buttons = new List<Button>();

		public void		setVisible(bool is_visible)
		{
			this.is_visible = is_visible;

			foreach(var button in this.buttons) {

				button.setVisible(this.is_visible);
			}

			if(!this.is_visible) {

				this.focused_button = null;
			}
		}
	};

	public List<Group>	groups;

	protected Group		default_group;

	public static float	FORCUSED_BUTTON_DEPTH = 0.1f;

	// ================================================================ //
	// MonoBehaviour からの継承.
	
	void	Awake()
	{
		this.groups = new List<Group>();

		this.default_group = new Group();

		this.default_group.name = "@default";

		this.groups.Add(this.default_group);
	}
	
	void	Start()
	{
	}
	
	void	Update()
	{
		//dbPrint.setLocate(1, 10);

		foreach(Group group in this.groups) {

			/*if(group.focused_button != null) {

				dbPrint.print(group.name + " " + group.focused_button.name.ToString(), 0.0f);

			} else {

				dbPrint.print(group.name, 0.0f);
			}*/

			if(!group.is_visible) {

				continue;
			}

			// 前のフレームでフォーカスしていたボタンを最初に実行.

			Button	next_focused = group.focused_button;

			if(group.focused_button != null) {

				group.focused_button.update(true);

				if(!group.focused_button.focused.current) {

					next_focused = null;
				}
			}

			// 他のボタンを実行.

			foreach(Button button in group.buttons) {

				if(button == group.focused_button) {

					continue;
				}

				button.update(next_focused == null);

				if(next_focused == null) {

					if(button.focused.current) {

						next_focused = button;
					}
				}
			}

			group.focused_button = next_focused;
		}
	}

	// ================================================================ //

	public Group	createGroup(string name)
	{
		Group	group = null;

		group = this.findGroup(name);

		if(group == null) {

			group      = new Group();
			group.name = name;

			this.groups.Add(group);
		}

		return(group);
	}

	public Group	findGroup(string name)
	{
		return(this.groups.Find(x => x.name == name));
	}

	// ボタンをつくる.
	public Button	createButton(string name, Texture texture, Vector2 size, string group_name)
	{
		GameObject		go = new GameObject();

		Button	button = go.AddComponent<Button>();

		button.create(texture, size);

		Group	group = this.findGroup(group_name);

		if(group == null) {

			group = this.default_group;
		}

		group.buttons.Add(button);

		button.name       = name;
		button.group_name = group.name;

		return(button);
	}
		
	// ================================================================ //
	// インスタンス.
	
	private	static ButtonRoot	instance = null;
	
	public static ButtonRoot	get()
	{
		if(ButtonRoot.instance == null) {
			
			ButtonRoot.instance = GameObject.Find("ButtonRoot").GetComponent<ButtonRoot>();
		}
		
		return(ButtonRoot.instance);
	}
	
}
