﻿using UnityEngine;
using System.Collections;
using MathExtension;

public class SlidePage : MonoBehaviour {

	public Sprite2DControl	sprite;

	protected int		div_count = 21;

	public static float		WIDTH  = 960.0f;
	public static float		HEIGHT = 720.0f;
	
	protected Vector2	size = new Vector2(WIDTH, HEIGHT);

	protected SlideTransitBasic	transit;

	// ================================================================ //
	// MonoBehaviour からの継承.

	void	Awake()
	{
	}

	void	Start()
	{
	}

	void	Update()
	{
		this.transit.update(Time.deltaTime);

		this.sprite.setVertexPositions(this.transit.positions);
	}

	// ================================================================ //

	public void		create(Texture texture, bool transition_effect)
	{
		//

		this.sprite = Sprite2DRoot.get().createSprite(texture, 21, false);
		this.sprite.setSize(this.size);

		this.transit = new SlideTransitBasic();
		this.transit.create(this.div_count, new Vector2(WIDTH, HEIGHT));

		if(!transition_effect) {

			this.transit.initial_positions.CopyTo(this.transit.positions, 0);
		}

		this.sprite.setVertexPositions(this.transit.positions);
	}
	
	public void		destroy()
	{
		this.sprite.destroy();
		
		GameObject.Destroy(this.gameObject);
	}
	
}

public class SlideTransitBasic {

	public SlideTransitBasic() {}

	protected int			div_count = 21;
	protected Vector2		size = new Vector2(640.0f, 480.0f);

	public Vector3[]		positions = null;
	public Vector3[]		initial_positions = null;
	public Vector3[]		velocities = null;

	public void		create(int div_count, Vector2 size)
	{
		this.div_count = div_count;
		this.size      = size;

		this.positions = Sprite2DRoot.get().calcVertexPositions(this.size.x, this.size.y, this.div_count);
		this.initial_positions = this.positions.Clone() as Vector3[];
		
		this.velocities = new Vector3[this.div_count*this.div_count];

		this.calc_round_shape();
	}

	public void		update(float delta_time)
	{
		int		div_count = this.div_count;

		//
		
		for(int y = 0;y < div_count;y++) {
			
			for(int x = 0;x < div_count;x++) {

				Vector3		position = this.positions[y*div_count + x];
				Vector3		restoration = Vector3.zero;

				restoration = this.initial_positions[y*div_count + x];

				this.velocities[y*div_count + x] *= 0.85f;
				this.velocities[y*div_count + x] += (restoration - position).normalized*Mathf.Pow((restoration - position).magnitude, 0.7f)*4.0f;

				if(Vector2.Distance(position, restoration) < 1.0f && this.velocities[y*div_count + x].magnitude < 1.0f) {
				
					this.positions[y*div_count + x] = restoration;
					this.velocities[y*div_count + x] = Vector2.zero;
				
				} else {

					this.positions[y*div_count + x] = position + this.velocities[y*div_count + x]*delta_time;
				}
			}
		}
	}

	protected void		calc_round_shape()
	{
		int		div_count = this.div_count;

		float	w = this.size.x;
		float	h = this.size.y;
		
		for(int y = 0;y < div_count;y++) {
			
			for(int x = 0;x < div_count;x++) {

				Vector2	position = this.positions[y*div_count + x];

				position.x *= h/w;

				float	radius = Mathf.Max(Mathf.Abs(position.x), Mathf.Abs(position.y));

				if(position.magnitude > 0.0f) {

					position *= radius*0.5f/position.magnitude;
				}

				this.positions[y*div_count + x] = position;
			}
		}
	}


} // 