﻿using UnityEngine;
using System.Collections;
using MathExtension;

public class SlidePlayer : MonoBehaviour {

	public static float		IDEAL_SCREEN_WIDTH  = 1280.0f;
	public static float		IDEAL_SCREEN_HEIGHT = 800.0f;

	protected Vector2	screen_size = new Vector2(IDEAL_SCREEN_WIDTH, IDEAL_SCREEN_HEIGHT);

	// ---------------------------------------------------------------- //

	public enum STEP {
		
		NONE = -1,
		
		TITLE = 0,			//
		SLIDE,
		THUMBNAIL,

		NUM,
	};
	Step<STEP>		step = new Step<STEP>(STEP.NONE);

	// ---------------------------------------------------------------- //

	public enum POINTER_TYPE {

		NONE = -1,

		YUBI = 0,
		SABA,
		ASUNAN,

		NUM,
	};
	protected POINTER_TYPE	pointer_type = POINTER_TYPE.ASUNAN;

	public GameObject[]		pointerPrefabs;
	public PointerBase[]	pointers;

	// ---------------------------------------------------------------- //
	
	public int					current_slide = 0;
	public Texture[]			slides;

	public SlidePage			prefab_page = null;

	protected SlidePage			page;
	protected bool				enable_transition_effect = true;

	public Texture				texture_back = null;					// 背景用　テクスチャー.
	public Texture				texture_button_next        = null;		// 次ページボタン　テクスチャー.
	public Texture				texture_button_previous    = null;		// 前ページボタン　テクスチャー.
	public Texture				texture_button_thumb       = null;		// サムネイルモードボタン　テクスチャー.
	public Texture				texture_button_transit     = null;		// ページ送り演出ボタン　オン状態　テクスチャー.
	public Texture				texture_button_transit_off = null;		// ページ送り演出ボタン　オフ状態　テクスチャー.
	
	protected Sprite2DControl	sprite_back   = null;

	protected Button			button_next;
	protected Button			button_previous;
	protected Button			button_thumb;
	protected Button			button_transit;
	
	protected Button[]			thumbs;
	protected Button[]			button_pointers;

	protected string	pressed_thumb = "";
	protected int		pressed_thumb_index = -1;

	// ================================================================ //
	// MonoBehaviour からの継承.

	void	Awake()
	{
		Sprite2DRoot.get().setViewportPixels(this.screen_size);

		// 背景.

		this.sprite_back = Sprite2DRoot.get().createSprite(this.texture_back, 2, false);
		this.sprite_back.setSize(this.screen_size);

		// カーソル.

		this.pointers = new PointerBase[(int)POINTER_TYPE.NUM];

		for(int i = 0;i < (int)POINTER_TYPE.NUM;i++) {

			if(this.pointerPrefabs[i] == null) {

				continue;
			}

			GameObject	go = GameObject.Instantiate(this.pointerPrefabs[i]) as GameObject;

			if(go == null) {

				continue;
			}

			this.pointers[i] = go.GetComponent<PointerBase>();
			this.pointers[i].create();
		}

		// ボタン.

		ButtonRoot.get().createGroup("tools");
		ButtonRoot.get().createGroup("thumbs");
		ButtonRoot.get().createGroup("cursors");

		Vector2	size     = new Vector2(96.0f, 96.0f);
		Vector2	position = new Vector2(this.screen_size.x/2.0f, this.screen_size.y/2.0f);
		float	margin = 32.0f;

		position.x -= size.x/2.0f + margin;
		position.y -= size.y/2.0f + margin;

		this.button_next = ButtonRoot.get().createButton("next", this.texture_button_next, size, "tools");
		this.button_next.sprite.setPosition(position);
		this.button_next.setVisible(false);

		position.y -= size.y + margin;
		this.button_previous = ButtonRoot.get().createButton("prev", this.texture_button_previous, size, "tools");
		this.button_previous.sprite.setPosition(position);
		this.button_previous.setVisible(false);

		position.y -= size.y + margin;
		this.button_thumb = ButtonRoot.get().createButton("thum", this.texture_button_thumb, size, "tools");
		this.button_thumb.sprite.setPosition(position);
		this.button_thumb.setVisible(false);

		position.y -= size.y + margin;
		this.button_transit = ButtonRoot.get().createButton("trans", this.texture_button_transit, size, "tools");
		this.button_transit.sprite.setPosition(position);
		this.button_transit.setVisible(false);
		this.button_transit.on_trigger_pressed = (name) =>
			{
				this.enable_transition_effect = !this.enable_transition_effect;

				if(this.enable_transition_effect) {

					this.button_transit.sprite.setTexture(this.texture_button_transit);

				} else {

					this.button_transit.sprite.setTexture(this.texture_button_transit_off);
				}
			};

		//

		position = new Vector2(-this.screen_size.x/2.0f, this.screen_size.y/2.0f);

		position.x += size.x/2.0f + margin;
		position.y -= size.y/2.0f + margin;

		this.button_pointers = new Button[(int)POINTER_TYPE.NUM];

		for(int i = 0;i < (int)POINTER_TYPE.NUM;i++) {

			this.button_pointers[i] = ButtonRoot.get().createButton(i.ToString(), this.pointers[i].texture_button, size, "cursors");
			this.button_pointers[i].sprite.setPosition(position);
			this.button_pointers[i].setVisible(false);

			position.y -= size.y + margin;
		}

		//

		this.change_cursor_type(POINTER_TYPE.NONE);
		this.pointer_type = POINTER_TYPE.YUBI;
	}

	void	Start()
	{
		//Screen.showCursor = false;

		this.create_thumbnails();

		this.step.set_next(STEP.TITLE);
 		//this.step.set_next(STEP.SLIDE);
	}

	void	Update()
	{

		// ---------------------------------------------------------------- //
		// 次の状態に移るかどうかを、チェックする.
		
		switch(this.step.do_transition()) {
			
			case STEP.TITLE:
			{
				if(Input.GetMouseButtonDown(0)) {

					ButtonRoot.get().findGroup("cursors").setVisible(true);
					this.change_cursor_type(this.pointer_type);
					this.step.set_next(STEP.SLIDE);
				}
			}
			break;

			case STEP.SLIDE:
			{
				if(this.button_thumb.pressed.trigger_on) {

					this.step.set_next(STEP.THUMBNAIL);
				}
			}
			break;

			case STEP.THUMBNAIL:
			{
				do {

					// モードチェンジボタン.
					if(this.button_thumb.pressed.trigger_on) {
	
						this.step.set_next(STEP.SLIDE);
						break;
					}

					// サムネイルがクリックされた.
					if(this.pressed_thumb_index >= 0) {

						this.current_slide = this.pressed_thumb_index;
						this.pressed_thumb_index = -1;
						this.step.set_next(STEP.SLIDE);
						break;
					}

				} while(false);

				if(this.step.get_next() != STEP.NONE) {

					ButtonRoot.get().findGroup("thumbs").setVisible(false);
				}
			}
			break;
		}
		
		// ---------------------------------------------------------------- //
		// 状態が遷移したときの初期化.
		
		while(this.step.get_next() != STEP.NONE) {
			
			switch(this.step.do_initialize()) {
				
				case STEP.TITLE:
				{
					ButtonRoot.get().findGroup("cursors").setVisible(false);
					this.sprite_back.setVertexColor(Color.white);
				}
				break;

				case STEP.SLIDE:
				{
					this.button_next.setVisible(true);
					this.button_previous.setVisible(true);
					this.button_thumb.setVisible(true);
					this.button_transit.setVisible(true);

					this.change_page_button_active();

					this.sprite_back.setVertexColor(new Color(0.7f, 0.7f, 0.7f));

					this.page = GameObject.Instantiate(this.prefab_page) as SlidePage;
			
					this.page.create(this.slides[this.current_slide], this.enable_transition_effect);
				}
				break;

				case STEP.THUMBNAIL:
				{
					this.button_next.setVisible(false);
					this.button_previous.setVisible(false);

					this.page.destroy();

					ButtonRoot.get().findGroup("thumbs").setVisible(true);
					this.pressed_thumb_index = -1;
				}
				break;
			}
		}
		
		// ---------------------------------------------------------------- //
		// 各状態での実行処理.
		
		switch(this.step.do_execution(Time.deltaTime)) {
			
			case STEP.SLIDE:
			{
				// ページ送りボタン.
				
				int		next_slide = this.current_slide;
		
				if(this.button_next.pressed.trigger_on) {
		
					next_slide++;
				}
				if(this.button_previous.pressed.trigger_on) {
					
					next_slide--;
				}

				next_slide = (int)Mathf.Clamp(next_slide, 0, this.slides.Length - 1);

				if(next_slide != this.current_slide) {
		
					this.page.destroy();
					
					this.page = GameObject.Instantiate(this.prefab_page) as SlidePage;
					this.page.create(this.slides[next_slide], this.enable_transition_effect);

					this.current_slide = next_slide;

					this.change_page_button_active();
				}
			}
			break;

			case STEP.THUMBNAIL:
			{
				this.pressed_thumb_index = -1;

				if(this.pressed_thumb != "") {

					if(!int.TryParse(this.pressed_thumb, out this.pressed_thumb_index)) {

						this.pressed_thumb_index = -1;
					}
				}
				this.pressed_thumb = "";
			}
			break;
		}

		// ---------------------------------------------------------------- //

		do {

			ButtonRoot.Group	group_cursors = ButtonRoot.get().findGroup("cursors");

			if(group_cursors.focused_button == null) {

				break;
			}
			if(!group_cursors.focused_button.pressed.trigger_on) {

				break;
			}

			int		next_type = (int)this.pointer_type;

			if(!int.TryParse(group_cursors.focused_button.name, out next_type)) {

				break;
			}

			if(next_type == (int)this.pointer_type) {

				break;
			}

			this.change_cursor_type((POINTER_TYPE)next_type);

		} while(false);

		if(this.pointer_type != POINTER_TYPE.NONE) {

			Vector2	cursor_position = Sprite2DRoot.get().convertMousePosition(Input.mousePosition.xy());

			this.pointers[(int)this.pointer_type].setPosition(cursor_position);
		}
	}

	// ================================================================ //

	public Vector2	getScreenSize()
	{
		return(this.screen_size);
	}

	// ================================================================ //

	protected void	change_page_button_active()
	{
		this.button_next.setActive(this.current_slide < this.slides.Length - 1);
		this.button_previous.setActive(this.current_slide > 0);
	}

	protected void	change_cursor_type(POINTER_TYPE next_type)
	{
		this.pointer_type = next_type;

		for(int i = 0;i < (int)POINTER_TYPE.NUM;i++) {

			this.pointers[i].setVisible((POINTER_TYPE)i == this.pointer_type);

			this.button_pointers[i].setActive((POINTER_TYPE)i != this.pointer_type);
		}
	}

	// サムネイルのスプライトを作る.
	protected void	create_thumbnails()
	{
		int		x_count = 8;
		
		this.thumbs = new Button[this.slides.Length];
		
		Vector2		thumnail_size = new Vector2(SlidePage.WIDTH/((float)x_count), SlidePage.HEIGHT/((float)x_count));
		
		Vector2		margin = new Vector2(4.0f, 4.0f);
		
		thumnail_size -= margin;

		for(int i = 0;i < this.slides.Length;i++) {
			
			this.thumbs[i] = ButtonRoot.get().createButton("thumb:" + i.ToString(), this.slides[i], thumnail_size, "thumbs");
			
			Vector2		position;
			
			position.x = -SlidePage.WIDTH/2.0f  + (i%x_count + 0.5f)*(thumnail_size.x + margin.x);
			position.y =  SlidePage.HEIGHT/2.0f - (i/x_count + 0.5f)*(thumnail_size.y + margin.y);
			
			this.thumbs[i].sprite.setPosition(position);
			this.thumbs[i].name = i.ToString();
			this.thumbs[i].on_trigger_pressed = (name) => this.pressed_thumb = name;

			this.thumbs[i].setVisible(false);
		}
	}

	// ================================================================ //
	// インスタンス.

	private	static SlidePlayer	instance = null;

	public static SlidePlayer	get()
	{
		if(SlidePlayer.instance == null) {

			SlidePlayer.instance = GameObject.Find("SlidePlayer").GetComponent<SlidePlayer>();
		}

		return(SlidePlayer.instance);
	}
}
