﻿using UnityEngine;
using System.Collections;
using MathExtension;

// 2D スプライト.
public class Sprite2DControl : MonoBehaviour {
	
	protected Vector2		size = Vector2.zero;

	// ================================================================ //
	// MonoBehaviour からの継承.

	void	Start()
	{
	}
	
	void	Update()
	{
	}

	// ================================================================ //

	// 位置をセットする.
	public void	setPosition(Vector2 position)
	{
		this.transform.localPosition = new Vector3(position.x, position.y, this.transform.localPosition.z);
	}

	public Vector2	getPosition()
	{
		return(this.transform.localPosition.xy());
	}

	public void	setDepth(float depth)
	{
		Vector3		position = this.transform.localPosition;

		position.z = depth;

		this.transform.localPosition = position;
	}

	// [degree] アングル（Z軸周りの回転）をセットする.
	public void	setAngle(float angle)
	{
		this.transform.localRotation = Quaternion.AngleAxis(angle, Vector3.forward);
	}

	// スケールをセットする.
	public void	setScale(Vector2 scale)
	{
		this.transform.localScale = new Vector3(scale.x, scale.y, 1.0f);
	}
	
	// サイズをセットする.
	public void		setSize(Vector2 size)
	{
		Sprite2DRoot.get().setSizeToSprite(this, size);
	}
	// サイズをゲットする.
	public Vector2 getSize()
	{
		return(this.size);
	}
	
	// 頂点カラーをセットする.
	public void		setVertexColor(Color color)
	{
		Sprite2DRoot.get().setVertexColorToSprite(this, color);
	}

	// テクスチャーをセットする.
	public void		setTexture(Texture texture)
	{
		this.GetComponent<MeshRenderer>().material.mainTexture = texture;
	}

	// マテリアルをセットする.
	public void		setMaterial(Material material)
	{
		this.GetComponent<MeshRenderer>().material = material;
	}
	// マテリアルをゲットする.
	public Material		getMaterial()
	{
		return(this.GetComponent<MeshRenderer>().material);
	}

	// ポイントがスプライトの上にある？.
	public bool		isContainPoint(Vector2 point)
	{
		bool	ret = false;

		Vector2		position = this.transform.localPosition.xy();
		Vector2		scale    = this.transform.localScale.xy();

		do {

			if(point.x < position.x - this.size.x/2.0f*scale.x || position.x + this.size.x/2.0f*scale.x < point.x) {

				break;
			}
			if(point.y < position.y - this.size.y/2.0f*scale.y || position.y + this.size.y/2.0f*scale.y < point.y) {

				break;
			}

			ret = true;

		} while(false);

		return(ret);
	}

	// 表示/非表示をセットする.
	public void		setVisible(bool is_visible)
	{
		this.GetComponent<MeshRenderer>().enabled = is_visible;
	}

	public void		setVertexPositions(Vector3[] positions)
	{
		Sprite2DRoot.get().setVertexPositionsToSprite(this, positions);
	}

	public int		getDivCount()
	{
		return(this.div_count);
	}

	public void		destroy()
	{
		GameObject.Destroy(this.gameObject);
	}

	// ================================================================ //
	// Sprite2DRoot よう.

	// サイズをセットする.
	public void	internalSetSize(Vector2 size)
	{
		this.size = size;
	}

	protected int	div_count = 2;

	public void	internalSetDivCount(int div_count)
	{
		this.div_count = div_count;
	}

}
